import glob
import re
import rpm
import unittest

M = None
ErlDrvDep = ""
ErlNifDep = ""

class TestAllMethods(unittest.TestCase):
    def test_sort_and_uniq(self):
        self.assertEqual(M.sort_and_uniq([1,2,2,2,2,4,3,2,1]), [1,2,3,4])

    def test_check_for_mfa(self):
        # This test requires erlang-erts RPM package installed
        filepath = glob.glob('/usr/lib*/erlang/lib/erts-*/ebin/erlang.beam')[0]
        self.assertEqual(M.check_for_mfa("%s/*/ebin" % M.ERLLIBDIR, {}, ('erlang', 'load_nif', 2)), filepath)

    def test_inspect_so_library_nif(self):
        # This test requires erlang-crypto RPM package installed
        filepath = glob.glob("/usr/lib*/erlang/lib/crypto-*/priv/lib/crypto.so")[0]
        self.assertEqual(M.inspect_so_library(filepath, 'nif_init', 'erlang(erl_nif_version)'), ErlNifDep)

    def test_inspect_so_library_drv(self):
        # This test requires erlang-erlsyslog RPM package installed
        filepath = glob.glob("/usr/lib*/erlang/lib/erlsyslog-*/priv/erlsyslog_drv.so")[0]
        self.assertEqual(M.inspect_so_library(filepath, 'driver_init', 'erlang(erl_drv_version)'), ErlDrvDep)

    def test_inspect_beam_file_arch(self):
        Deps = ['erlang-erts(x86-64)', 'erlang-kernel(x86-64)', 'erlang-stdlib(x86-64)']
        self.assertEqual(M.inspect_beam_file('x86-64', "./test.beam"), Deps)

    def test_inspect_beam_file_noarch(self):
        Deps = ['erlang-erts', 'erlang-kernel', 'erlang-stdlib']
        self.assertEqual(M.inspect_beam_file('noarch', "./test.beam"), Deps)

    def test_check_for_absense_of_buildarch_macro(self):
        self.assertEqual(rpm.expandMacro("%{buildarch}"), "%{buildarch}")

    def test_check_for_target_cpu_macro(self):
        self.assertNotEqual(rpm.expandMacro("%{_target_cpu}"), "%{_target_cpu}")

if __name__ == "__main__":
    M = __import__("erlang-find-requires")

    ts = rpm.TransactionSet()
    mi = ts.dbMatch('name', "erlang-erts")
    h = next(mi)
    ds = dict(map(lambda x: x[0].split(" ")[1::2], h.dsFromHeader('providename')))
    ErlDrvDep = "erlang(erl_drv_version) = %s" % ds['erlang(erl_drv_version)']
    ErlNifDep = "erlang(erl_nif_version) = %s" % ds['erlang(erl_nif_version)']

    prog = re.compile("/usr/lib(64)?/erlang/lib")
    M.ERLLIBDIR = prog.match(glob.glob("/usr/lib*/erlang/lib/erts-*/ebin/erts.app")[0])[0]

    unittest.main()



#!/usr/bin/python3

# Copyright (c) 2016 Peter Lemenkov <lemenkov@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# This script reads filenames from STDIN and outputs any relevant provides
# information that needs to be included in the package.

import getopt
import glob
import pybeam
import re
import sys

BUILDDIR=""

opts, args = getopt.getopt(sys.argv[1:],"b:",["builddir="])

for opt, arg in opts:
	if opt in ("-b", "--builddir"):
		BUILDDIR=arg

# All the files and directories from the package mentioned in erlang.attr as
# %__erlang_path
rawcontent = sys.stdin.readlines()

# See $BUILDDIR/erts/emulator/*/erl_bif_list.h
ErtsBIFProvides = [
	"erlang(erlang:*/2)",
	"erlang(erlang:++/2)",
	"erlang(erlang:+/1)",
	"erlang(erlang:+/2)",
	"erlang(erlang:--/2)",
	"erlang(erlang:-/1)",
	"erlang(erlang:-/2)",
	"erlang(erlang://2)",
	"erlang(erlang:/=/2)",
	"erlang(erlang:</2)",
	"erlang(erlang:=/=/2)",
	"erlang(erlang:=:=/2)",
	"erlang(erlang:=</2)",
	"erlang(erlang:==/2)",
	"erlang(erlang:>/2)",
	"erlang(erlang:>=/2)",
	"erlang(erlang:and/2)",
	"erlang(erlang:band/2)",
	"erlang(erlang:bnot/1)",
	"erlang(erlang:bor/2)",
	"erlang(erlang:bsl/2)",
	"erlang(erlang:bsr/2)",
	"erlang(erlang:bxor/2)",
	"erlang(erlang:div/2)",
	"erlang(erlang:not/1)",
	"erlang(erlang:or/2)",
	"erlang(erlang:rem/2)",
	"erlang(erlang:xor/2)"
]
HipeBIFprovides = [
	"erlang(hipe_bifs:add_ref/2)",
	"erlang(hipe_bifs:alloc_data/2)",
	"erlang(hipe_bifs:array/2)",
	"erlang(hipe_bifs:array_length/1)",
	"erlang(hipe_bifs:array_sub/2)",
	"erlang(hipe_bifs:array_update/3)",
	"erlang(hipe_bifs:atom_to_word/1)",
	"erlang(hipe_bifs:bif_address/3)",
	"erlang(hipe_bifs:bitarray/2)",
	"erlang(hipe_bifs:bitarray_sub/2)",
	"erlang(hipe_bifs:bitarray_update/3)",
	"erlang(hipe_bifs:bytearray/2)",
	"erlang(hipe_bifs:bytearray_sub/2)",
	"erlang(hipe_bifs:bytearray_update/3)",
	"erlang(hipe_bifs:call_count_clear/1)",
	"erlang(hipe_bifs:call_count_get/1)",
	"erlang(hipe_bifs:call_count_off/1)",
	"erlang(hipe_bifs:call_count_on/1)",
	"erlang(hipe_bifs:check_crc/1)",
	"erlang(hipe_bifs:code_size/1)",
	"erlang(hipe_bifs:constants_size/0)",
	"erlang(hipe_bifs:debug_native_called/2)",
	"erlang(hipe_bifs:enter_code/2)",
	"erlang(hipe_bifs:enter_sdesc/1)",
	"erlang(hipe_bifs:find_na_or_make_stub/2)",
	"erlang(hipe_bifs:fun_to_address/1)",
	"erlang(hipe_bifs:gc_info/0)",
	"erlang(hipe_bifs:gc_info_clear/0)",
	"erlang(hipe_bifs:gc_timer/0)",
	"erlang(hipe_bifs:gc_timer_clear/0)",
	"erlang(hipe_bifs:get_fe/2)",
	"erlang(hipe_bifs:get_hrvtime/0)",
	"erlang(hipe_bifs:get_rts_param/1)",
	"erlang(hipe_bifs:incremental_gc_info/0)",
	"erlang(hipe_bifs:invalidate_funinfo_native_addresses/1)",
	"erlang(hipe_bifs:in_native/0)",
	"erlang(hipe_bifs:llvm_fix_pinned_regs/0)",
	"erlang(hipe_bifs:mark_referred_from/1)",
	"erlang(hipe_bifs:merge_term/1)",
	"erlang(hipe_bifs:message_info/0)",
	"erlang(hipe_bifs:message_info_clear/0)",
	"erlang(hipe_bifs:message_sizes/0)",
	"erlang(hipe_bifs:misc_timer/0)",
	"erlang(hipe_bifs:misc_timer_clear/0)",
	"erlang(hipe_bifs:modeswitch_debug_off/0)",
	"erlang(hipe_bifs:modeswitch_debug_on/0)",
	"erlang(hipe_bifs:nstack_used_size/0)",
	"erlang(hipe_bifs:patch_call/3)",
	"erlang(hipe_bifs:patch_insn/3)",
	"erlang(hipe_bifs:pause_times/0)",
	"erlang(hipe_bifs:primop_address/1)",
	"erlang(hipe_bifs:process_info/0)",
	"erlang(hipe_bifs:process_info_clear/0)",
	"erlang(hipe_bifs:redirect_referred_from/1)",
	"erlang(hipe_bifs:ref/1)",
	"erlang(hipe_bifs:ref_get/1)",
	"erlang(hipe_bifs:ref_set/2)",
	"erlang(hipe_bifs:remove_refs_from/1)",
	"erlang(hipe_bifs:send_timer/0)",
	"erlang(hipe_bifs:send_timer_clear/0)",
	"erlang(hipe_bifs:set_funinfo_native_address/3)",
	"erlang(hipe_bifs:set_native_address/3)",
	"erlang(hipe_bifs:set_native_address_in_fe/2)",
	"erlang(hipe_bifs:shared_gc_info/0)",
	"erlang(hipe_bifs:shared_gc_timer/0)",
	"erlang(hipe_bifs:show_estack/1)",
	"erlang(hipe_bifs:show_heap/1)",
	"erlang(hipe_bifs:show_nstack/1)",
	"erlang(hipe_bifs:show_pcb/1)",
	"erlang(hipe_bifs:show_term/1)",
	"erlang(hipe_bifs:stop_hrvtime/0)",
	"erlang(hipe_bifs:system_crc/0)",
	"erlang(hipe_bifs:system_timer/0)",
	"erlang(hipe_bifs:system_timer_clear/0)",
	"erlang(hipe_bifs:term_to_word/1)",
	"erlang(hipe_bifs:trap_count_clear/0)",
	"erlang(hipe_bifs:trap_count_get/0)",
	"erlang(hipe_bifs:update_code_size/3)",
	"erlang(hipe_bifs:write_u32/2)",
	"erlang(hipe_bifs:write_u64/2)",
	"erlang(hipe_bifs:write_u8/2)"
]

Provides = []

rawcontent = list(map(lambda x: x.rstrip('\n'), rawcontent))

# Check for a specific cases
appmask = re.compile(".*/ebin/.*\.app")
# There should be only one app-file or none
for appfile in sorted([p for p in rawcontent if appmask.match(p)]):
	f = open(appfile, 'r')
	appcontents = f.read()
	f.close()
	# module erlang-erts (Erlang VM)
	if appcontents.split(",")[1].lstrip().rstrip() == "erts":
		# Export DRV version
		f = open("%s/erts/emulator/beam/erl_driver.h" % BUILDDIR, 'r')
		ERL_DRV_EXTENDED_MAJOR_VERSION = None
		ERL_DRV_EXTENDED_MINOR_VERSION = None
		for line in f:
			ms = re.search("(?<=^#define\sERL_DRV_EXTENDED_MAJOR_VERSION\s)[0-9]+(?=$)", line)
			if ms:
				ERL_DRV_EXTENDED_MAJOR_VERSION = ms.group(0)
		for line in f:
			ms = re.search("(?<=^#define\sERL_DRV_EXTENDED_MINOR_VERSION\s)[0-9]+(?=$)", line)
			if ms:
				ERL_DRV_EXTENDED_MINOR_VERSION = ms.group(0)
		# FIXME die here if ERL_DRV_EXTENDED_MAJOR_VERSION or ERL_DRV_EXTENDED_MINOR_VERSION is None
		Provides += "erlang(erl_drv_version) = %s.%s" % (ERL_DRV_EXTENDED_MAJOR_VERSION, ERL_DRV_EXTENDED_MINOR_VERSION)
		f.close()

		# Export NIF version
		f = open("%s/erts/emulator/beam/erl_nif.h" % BUILDDIR, 'r')
		ERL_NIF_MAJOR_VERSION = None
		ERL_NIF_MINOR_VERSION = None
		for line in f:
			ms = re.search("(?<=^#define\sERL_NIF_MAJOR_VERSION\s)[0-9]+(?=$)", line)
			if ms:
				ERL_NIF_MAJOR_VERSION = ms.group(0)
		for line in f:
			ms = re.search("(?<=^#define\sERL_NIF_MINOR_VERSION\s)[0-9]+(?=$)", line)
			if ms:
				ERL_NIF_MINOR_VERSION = ms.group(0)
		# FIXME die here if ERL_NIF_MAJOR_VERSION or ERL_NIF_MINOR_VERSION is None
		Provides += "erlang(erl_drv_version) = %s.%s" % (ERL_NIF_MAJOR_VERSION, ERL_NIF_MINOR_VERSION)
		f.close()

		# Add BIFs for erlang:F/A
		Provides += ErtsBIFProvides

	# module erlang-hipe
	if appcontents.split(",")[1].lstrip().rstrip() == "hipe":
		# Add BIFs for hipe_bifs:F/A
		Provides += HipeBIFprovides

# Iterate over all BEAM-files
beammask = re.compile(".*/ebin/.*\.beam")
for package in sorted([p for p in rawcontent if beammask.match(p)]):
        b = pybeam.BeamFile(package)
	# Two special cases:
	# * eunit_test - add "erlang(eunit_test:nonexisting_function/0)"
	# * wx - add "erlang(demo:start/0)"
	Provides += list(map(lambda x: 'erlang(%s:%s/%d)' % (b.modulename,x[0],x[1]), b.exports))

for prov in sorted(Provides):
	print(prov)



#!/usr/bin/python3

# Copyright (c) 2016,2017 Peter Lemenkov <lemenkov@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# This script reads filenames from STDIN and outputs any relevant requires
# information that needs to be included in the package.

import argparse
import glob
import pybeam
import re
import rpm
import sys

from elftools.elf.elffile import ELFFile

# Globals
ERLLIBDIR = ""
ERLSHRDIR = "/usr/share/erlang/lib"

# See $BUILDROOT/erts/emulator/*/erl_bif_list.h
# erlang:F/A
ErtsBIFProvides = [
	("*", 2, 0),
	("++", 2, 0),
	("+", 1, 0),
	("+", 2, 0),
	("--", 2, 0),
	("-", 1, 0),
	("-", 2, 0),
	("/", 2, 0),
	("/=", 2, 0),
	("<", 2, 0),
	("=/=", 2, 0),
	("=:=", 2, 0),
	("=<", 2, 0),
	("==", 2, 0),
	(">", 2, 0),
	(">=", 2, 0),
	("and", 2, 0),
	("band", 2, 0),
	("bnot", 1, 0),
	("bor", 2, 0),
	("bsl", 2, 0),
	("bsr", 2, 0),
	("bxor", 2, 0),
	("div", 2, 0),
	("not", 1, 0),
	("or", 2, 0),
	("rem", 2, 0),
	("xor", 2, 0)
]

# See $BUILDROOT/erts/emulator/*/erl_bif_list.h
# hipe_bifs:F/A
HipeBIFSprovides = [
	("add_ref", 2, 0),
	("alloc_data", 2, 0),
	("array", 2, 0),
	("array_length", 1, 0),
	("array_sub", 2, 0),
	("array_update", 3, 0),
	("atom_to_word", 1, 0),
	("bif_address", 3, 0),
	("bitarray", 2, 0),
	("bitarray_sub", 2, 0),
	("bitarray_update", 3, 0),
	("bytearray", 2, 0),
	("bytearray_sub", 2, 0),
	("bytearray_update", 3, 0),
	("call_count_clear", 1, 0),
	("call_count_get", 1, 0),
	("call_count_off", 1, 0),
	("call_count_on", 1, 0),
	("check_crc", 1, 0),
	("code_size", 1, 0),
	("constants_size", 0, 0),
	("debug_native_called", 2, 0),
	("enter_code", 2, 0),
	("enter_sdesc", 1, 0),
	("find_na_or_make_stub", 2, 0),
	("fun_to_address", 1, 0),
	("gc_info", 0, 0),
	("gc_info_clear", 0, 0),
	("gc_timer", 0, 0),
	("gc_timer_clear", 0, 0),
	("get_fe", 2, 0),
	("get_hrvtime", 0, 0),
	("get_rts_param", 1, 0),
	("incremental_gc_info", 0, 0),
	("invalidate_funinfo_native_addresses", 1, 0),
	("in_native", 0, 0),
	("llvm_fix_pinned_regs", 0, 0),
	("mark_referred_from", 1, 0),
	("merge_term", 1, 0),
	("message_info", 0, 0),
	("message_info_clear", 0, 0),
	("message_sizes", 0, 0),
	("misc_timer", 0, 0),
	("misc_timer_clear", 0, 0),
	("modeswitch_debug_off", 0, 0),
	("modeswitch_debug_on", 0, 0),
	("nstack_used_size", 0, 0),
	("patch_call", 3, 0),
	("patch_insn", 3, 0),
	("pause_times", 0, 0),
	("primop_address", 1, 0),
	("process_info", 0, 0),
	("process_info_clear", 0, 0),
	("redirect_referred_from", 1, 0),
	("ref", 1, 0),
	("ref_get", 1, 0),
	("ref_set", 2, 0),
	("remove_refs_from", 1, 0),
	("send_timer", 0, 0),
	("send_timer_clear", 0, 0),
	("set_funinfo_native_address", 3, 0),
	("set_native_address", 3, 0),
	("set_native_address_in_fe", 2, 0),
	("shared_gc_info", 0, 0),
	("shared_gc_timer", 0, 0),
	("show_estack", 1, 0),
	("show_heap", 1, 0),
	("show_nstack", 1, 0),
	("show_pcb", 1, 0),
	("show_term", 1, 0),
	("stop_hrvtime", 0, 0),
	("system_crc", 0, 0),
	("system_timer", 0, 0),
	("system_timer_clear", 0, 0),
	("term_to_word", 1, 0),
	("trap_count_clear", 0, 0),
	("trap_count_get", 0, 0),
	("update_code_size", 3, 0),
	("write_u32", 2, 0),
	("write_u64", 2, 0),
	("write_u8", 2, 0)
]

# fastest sort + uniq
# see http://www.peterbe.com/plog/uniqifiers-benchmark
def sort_and_uniq(List):
	return list(set(List))

def check_for_mfa(Path, Dict, MFA):
	(M, F, A) = MFA
	Provides = []
        #  First we try to find a (list of) module(s)...
	Beams = glob.glob("%s/%s.beam" % (Path, M))
	if Beams != []:
		# ...and we'll use a first match, e.g. Beams[0] (as Erlang VM will do).
                # But before parsing module let's check if we already parsed
                # it, and stored the results in a dict.
		Provides = Dict.get(Beams[0])
		if not Provides:
			# No, we have to parse beam-file for the first time.
			b = pybeam.BeamFile(Beams[0])
			Provides = b.exports
			# Note - there are two special cases:
			# * eunit_test - add "erlang(eunit_test:nonexisting_function/0)"
			# * wx - add "erlang(demo:start/0)"
			if M == "erlang":
				Provides += ErtsBIFProvides
			Dict[Beams[0]] = Provides

                # Now Provides contains module's M export table. Let's check if
                # this module M actually exports a required function F with
                # arity A.
		for (F0, A0, Idx) in Provides:
			if F0 == F and A0 == A:
				# Always return first match. See comment above.
				return Beams[0]

	return None

def inspect_so_library(library, export_name, dependency_name):
    with open(library, 'rb') as f:
        elffile = ELFFile(f)
        dynsym = elffile.get_section_by_name('.dynsym')
        for sym in dynsym.iter_symbols():
            if sym.name == export_name:
                ts = rpm.TransactionSet()
                mi = ts.dbMatch('providename', dependency_name)
                h = next(mi)
                ds = dict(map(lambda x: x[0].split(" ")[1::2], h.dsFromHeader('providename')))
                if dependency_name in ds:
                    f.close()
                    return "%s = %s" % (dependency_name, ds[dependency_name])

        f.close()
        return None


# Compatibility function for the upcoming RPM Python3 API
# See https://bugzilla.redhat.com/1693771
def b2s(data):
    if isinstance(data, bytes):
        return data.decode('utf-8')
    return data

def inspect_beam_file(ISA, filename):
    b = pybeam.BeamFile(filename)
    # [(M,F,A),...]
    BeamMFARequires = sort_and_uniq(b.imports)

    Dict = {}
    # Filter out locally provided Requires

    # dirname(filename) could be:
    # * '$BUILDROOT/elixir-1.4.2-1.fc26.noarch/usr/share/elixir/1.4.2/lib/mix/ebin'
    # * '$BUILDROOT/erlang-y-combinator-1.0-1.fc26.noarch/usr/lib/erlang/lib/y-1.0/ebin'
    # * '$BUILDROOT/erlang-emmap-0-0.18.git05ae1bb.fc26.x86_64/usr/lib64/erlang/lib/emmap-0/ebin'
    # WARNING - this won't work for files from ERLLIBDIR
    BeamMFARequires = list(filter(lambda X: check_for_mfa('/'.join(filename.split('/')[:-3] + ["*", "ebin"]), Dict, X) is None, BeamMFARequires))

    Dict = {}
    # TODO let's find modules which provides these requires
    for (M,F,A) in BeamMFARequires:
        # FIXME check in noarch Erlang dir also
        if not check_for_mfa("%s/*/ebin" % ERLLIBDIR, Dict, (M, F, A)) and not check_for_mfa("%s/*/ebin" % ERLSHRDIR, Dict, (M, F, A)):
            print("ERROR: Cant find %s:%s/%d while processing '%s'" % (M,F,A, filename), file=sys.stderr)
            # We shouldn't stop further processing here - let pretend this is just a warning
            #exit(1)

    BeamModRequires = sort_and_uniq(Dict.keys())

    # let's find RPM-packets to which these modules belongs
    # We return more than one match since there could be situations where the same
    # object belongs to more than one package.
    ts = rpm.TransactionSet()
    RPMRequires = [item for sublist in map(
            lambda x: [(b2s(h[rpm.RPMTAG_NAME]), b2s(h[rpm.RPMTAG_ARCH])) for h in ts.dbMatch('basenames', x)],
            BeamModRequires
        ) for item in sublist]

    Ret = []
    for (req, PkgISA) in sort_and_uniq(RPMRequires):
        # ISA == "" if rpmbuild invoked with --target noarch
        if ISA == "noarch" or ISA == "" or PkgISA == "noarch":
            # noarch package - we don't care about arch dependency
            # erlang-erts erlang-kernel ...
            Ret += ["%s" % req]
        else:
            # arch-dependent package - we will use exact arch of adependent packages
            # erlang-erts(x86-64) erlang-kernel(x86-64) ...
            Ret += ["%s(%s)" % (req, ISA)]

    return sorted(Ret)

if __name__ == "__main__":

    ##
    ## Begin
    ##

    parser = argparse.ArgumentParser()

    # Get package's ISA
    parser.add_argument("-i", "--isa", nargs='?')
    args = parser.parse_args()

    if args.isa:
        # Convert "(x86-64)" to "x86-64"
        ISA=args.isa[1:-1]
    else:
        ISA="noarch"

    # Get the main Erlang directory
    prog = re.compile("/usr/lib(64)?/erlang/lib")
    ERLLIBDIR = prog.match(glob.glob("/usr/lib*/erlang/lib/erts-*/ebin/erts.app")[0])[0]

    # All the Erlang files matched by erlang.attr specification from the
    # package. Modern RPM version passes files one by one (a list
    # containing one filename prefixed by '\n'. We do not support older RPM
    # versions.
    #
    # We read filename as a list with a single element from stdin, get the
    # first element in the list, strip off the prefix, and pass it into the
    # main function.
    filename = sys.stdin.readlines()[0].rstrip('\n')

    Ret = []
    if filename.endswith(".beam"):
        Ret = inspect_beam_file(ISA, filename)

    elif filename.endswith(".so"):
        Ret += [inspect_so_library(filename, 'nif_init', 'erlang(erl_nif_version)')]
        Ret += [inspect_so_library(filename, 'driver_init', 'erlang(erl_drv_version)')]

    elif filename.endswith(".app"):
        # TODO we don't know what to do with *.app files yet
        pass

    else:
        # Unknown type
        pass

    for StringDependency in Ret:
        if StringDependency != None:
            print(StringDependency)
